@@ -1,7 +1,8 @@
 import asyncio
+import json
 import logging
 import re
-from typing import List
+from typing import List, Optional
 from asyncio_mqtt import Client
 from paho.mqtt.client import MQTTMessage
 from bluetti_mqtt.bus import CommandMessage, EventBus, ParserMessage
@@ -22,13 +23,26 @@ COMMAND_TOPIC_RE = re.compile(r'^bluetti/command/(\w+)-(\d+)/([a-z_]+)$')
 class MQTTClient:
     message_queue: asyncio.Queue
 
-    def __init__(self, host: str, devices: List[BluettiDevice], bus: EventBus):
-        self.host = host
+    def __init__(
+        self,
+        devices: List[BluettiDevice],
+        bus: EventBus,
+        hostname: str,
+        port: int = 1883,
+        username: Optional[str] = None,
+        password: Optional[str] = None,
+    ):
         self.devices = devices
         self.bus = bus
+        self.hostname = hostname
+        self.port = port
+        self.username = username
+        self.password = password
 
     async def run(self):
-        async with Client(self.host) as client:
+        async with Client(hostname=self.hostname, port=self.port, username=self.username, password=self.password) as client:
+            logging.info('Connected to MQTT broker...')
+
             # Connect to event bus
             self.message_queue = asyncio.Queue()
             self.bus.add_parser_listener(self.handle_message)
@@ -88,7 +102,7 @@ class MQTTClient:
             mode = AutoSleepMode[str(mqtt_message.payload)]
             cmd = UpdateFieldCommand(0x0B, 0xF5, mode.value)
         else:
-            logging.debug(f'Recevied command for unknown topic: {m[3]} - {mqtt_message.topic}')
+            logging.warn(f'Recevied command for unknown topic: {m[3]} - {mqtt_message.topic}')
             return
 
         await self.bus.put(CommandMessage(device, cmd))
@@ -130,14 +144,66 @@ class MQTTClient:
                 topic_prefix + 'ac_output_mode',
                 payload=msg.parser.ac_output_mode.name.encode()
             )
+            await client.publish(
+                topic_prefix + 'internal_ac_voltage',
+                payload=str(msg.parser.internal_ac_voltage).encode()
+            )
+            await client.publish(
+                topic_prefix + 'internal_current_one',
+                payload=str(msg.parser.internal_current_one).encode()
+            )
+            await client.publish(
+                topic_prefix + 'internal_power_one',
+                payload=str(msg.parser.internal_power_one).encode()
+            )
+            await client.publish(
+                topic_prefix + 'internal_ac_frequency',
+                payload=str(msg.parser.internal_ac_frequency).encode()
+            )
+            await client.publish(
+                topic_prefix + 'internal_current_two',
+                payload=str(msg.parser.internal_current_two).encode()
+            )
+            await client.publish(
+                topic_prefix + 'internal_power_two',
+                payload=str(msg.parser.internal_power_two).encode()
+            )
             await client.publish(
                 topic_prefix + 'ac_input_voltage',
                 payload=str(msg.parser.ac_input_voltage).encode()
             )
+            await client.publish(
+                topic_prefix + 'internal_current_three',
+                payload=str(msg.parser.internal_current_three).encode()
+            )
+            await client.publish(
+                topic_prefix + 'internal_power_three',
+                payload=str(msg.parser.internal_power_three).encode()
+            )
             await client.publish(
                 topic_prefix + 'ac_input_frequency',
                 payload=str(msg.parser.ac_input_frequency).encode()
             )
+            await client.publish(
+                topic_prefix + 'dc_input_voltage1',
+                payload=str(msg.parser.dc_input_voltage).encode()
+            )
+            await client.publish(
+                topic_prefix + 'dc_input_power1',
+                payload=str(msg.parser.dc_input_power).encode()
+            )
+            await client.publish(
+                topic_prefix + 'dc_input_current1',
+                payload=str(msg.parser.dc_input_current).encode()
+            )
+            pack_details = {
+                'percent': msg.parser.pack_battery_percent,
+                'voltages': [float(d) for d in msg.parser.pack_voltages],
+            }
+            await client.publish(
+                topic_prefix + f'pack_details{msg.parser.pack_num}',
+                payload=json.dumps(pack_details, separators=(',', ':')).encode()
+            )
         elif isinstance(msg.parser, ControlPageParser):
             await client.publish(
                 topic_prefix + 'ups_mode',
